@@ -1,12 +1,21 @@
 import asyncio
 import sys
-from typing import Any, NoReturn
+from typing import Any, List, NoReturn, Optional
 from unittest import mock
 
 import pytest
 
 import aiohttp
-from aiohttp import ClientConnectionResetError, ServerTimeoutError, WSMsgType, hdrs, web
+from aiohttp import (
+    ClientConnectionResetError,
+    ServerTimeoutError,
+    WSMessageTypeError,
+    WSMsgType,
+    hdrs,
+    web,
+)
+from aiohttp._websocket.reader import WebSocketDataQueue
+from aiohttp.client_ws import ClientWSTimeout
 from aiohttp.http import WSCloseCode
 from aiohttp.pytest_plugin import AiohttpClient
 
@@ -16,8 +25,12 @@ else:
     import async_timeout
 
 
-async def test_send_recv_text(aiohttp_client) -> None:
-    async def handler(request):
+class PatchableWebSocketDataQueue(WebSocketDataQueue):
+    """A WebSocketDataQueue that can be patched."""
+
+
+async def test_send_recv_text(aiohttp_client: AiohttpClient) -> None:
+    async def handler(request: web.Request) -> web.WebSocketResponse:
         ws = web.WebSocketResponse()
         await ws.prepare(request)
 
@@ -57,7 +70,28 @@ async def test_send_recv_bytes_bad_type(aiohttp_client) -> None:
     resp = await client.ws_connect("/")
     await resp.send_str("ask")
 
-    with pytest.raises(TypeError):
+    with pytest.raises(WSMessageTypeError):
+        await resp.receive_bytes()
+        await resp.close()
+
+
+async def test_recv_bytes_after_close(aiohttp_client: AiohttpClient) -> None:
+    async def handler(request: web.Request) -> NoReturn:
+        ws = web.WebSocketResponse()
+        await ws.prepare(request)
+
+        await ws.close()
+        assert False
+
+    app = web.Application()
+    app.router.add_route("GET", "/", handler)
+    client = await aiohttp_client(app)
+    resp = await client.ws_connect("/")
+
+    with pytest.raises(
+        WSMessageTypeError,
+        match=f"Received message {WSMsgType.CLOSE}:.+ is not WSMsgType.BINARY",
+    ):
         await resp.receive_bytes()
         await resp.close()
 
@@ -102,14 +136,35 @@ async def test_send_recv_text_bad_type(aiohttp_client) -> None:
 
     await resp.send_bytes(b"ask")
 
-    with pytest.raises(TypeError):
+    with pytest.raises(WSMessageTypeError):
         await resp.receive_str()
 
         await resp.close()
 
 
-async def test_send_recv_json(aiohttp_client) -> None:
-    async def handler(request):
+async def test_recv_text_after_close(aiohttp_client: AiohttpClient) -> None:
+    async def handler(request: web.Request) -> NoReturn:
+        ws = web.WebSocketResponse()
+        await ws.prepare(request)
+
+        await ws.close()
+        assert False
+
+    app = web.Application()
+    app.router.add_route("GET", "/", handler)
+    client = await aiohttp_client(app)
+    resp = await client.ws_connect("/")
+
+    with pytest.raises(
+        WSMessageTypeError,
+        match=f"Received message {WSMsgType.CLOSE}:.+ is not WSMsgType.TEXT",
+    ):
+        await resp.receive_str()
+        await resp.close()
+
+
+async def test_send_recv_json(aiohttp_client: AiohttpClient) -> None:
+    async def handler(request: web.Request) -> web.WebSocketResponse:
         ws = web.WebSocketResponse()
         await ws.prepare(request)
 
@@ -130,7 +185,29 @@ async def test_send_recv_json(aiohttp_client) -> None:
     await resp.close()
 
 
-async def test_ping_pong(aiohttp_client) -> None:
+async def test_send_recv_frame(aiohttp_client: AiohttpClient) -> None:
+    async def handler(request: web.Request) -> web.WebSocketResponse:
+        ws = web.WebSocketResponse()
+        await ws.prepare(request)
+
+        data = await ws.receive()
+        await ws.send_frame(data.data, data.type)
+        await ws.close()
+        return ws
+
+    app = web.Application()
+    app.router.add_route("GET", "/", handler)
+    client = await aiohttp_client(app)
+    resp = await client.ws_connect("/")
+    await resp.send_frame(b"test", WSMsgType.BINARY)
+
+    data = await resp.receive()
+    assert data.data == b"test"
+    assert data.type is WSMsgType.BINARY
+    await resp.close()
+
+
+async def test_ping_pong(aiohttp_client: AiohttpClient) -> None:
     loop = asyncio.get_event_loop()
     closed = loop.create_future()
 
@@ -238,7 +315,6 @@ async def test_concurrent_close(aiohttp_client) -> None:
     client_ws = None
 
     async def handler(request):
-        nonlocal client_ws
         ws = web.WebSocketResponse()
         await ws.prepare(request)
 
@@ -394,7 +470,7 @@ async def test_close_manual(aiohttp_client) -> None:
     assert resp.closed
 
 
-async def test_close_timeout(aiohttp_client) -> None:
+async def test_close_timeout_sock_close_read(aiohttp_client) -> None:
     async def handler(request):
         ws = web.WebSocketResponse()
         await ws.prepare(request)
@@ -406,7 +482,39 @@ async def test_close_timeout(aiohttp_client) -> None:
     app = web.Application()
     app.router.add_route("GET", "/", handler)
     client = await aiohttp_client(app)
-    resp = await client.ws_connect("/", timeout=0.2, autoclose=False)
+    timeout = ClientWSTimeout(ws_close=0.2)
+    resp = await client.ws_connect("/", timeout=timeout, autoclose=False)
+
+    await resp.send_bytes(b"ask")
+
+    msg = await resp.receive()
+    assert msg.data == "test"
+    assert msg.type == aiohttp.WSMsgType.TEXT
+
+    msg = await resp.close()
+    assert resp.closed
+    assert isinstance(resp.exception(), asyncio.TimeoutError)
+
+
+async def test_close_timeout_deprecated(aiohttp_client) -> None:
+    async def handler(request):
+        ws = web.WebSocketResponse()
+        await ws.prepare(request)
+        await ws.receive_bytes()
+        await ws.send_str("test")
+        await asyncio.sleep(1)
+        return ws
+
+    app = web.Application()
+    app.router.add_route("GET", "/", handler)
+    client = await aiohttp_client(app)
+    with pytest.warns(
+        DeprecationWarning,
+        match="parameter 'timeout' of type 'float' "
+        "is deprecated, please use "
+        r"'timeout=ClientWSTimeout\(ws_close=...\)'",
+    ):
+        resp = await client.ws_connect("/", timeout=0.2, autoclose=False)
 
     await resp.send_bytes(b"ask")
 
@@ -535,7 +643,7 @@ async def test_recv_timeout(aiohttp_client) -> None:
     await resp.close()
 
 
-async def test_receive_timeout(aiohttp_client) -> None:
+async def test_receive_timeout_sock_read(aiohttp_client) -> None:
     async def handler(request):
         ws = web.WebSocketResponse()
         await ws.prepare(request)
@@ -547,10 +655,38 @@ async def test_receive_timeout(aiohttp_client) -> None:
     app.router.add_route("GET", "/", handler)
 
     client = await aiohttp_client(app)
-    resp = await client.ws_connect("/", receive_timeout=0.1)
+    receive_timeout = ClientWSTimeout(ws_receive=0.1)
+    resp = await client.ws_connect("/", timeout=receive_timeout)
 
     with pytest.raises(asyncio.TimeoutError):
-        await resp.receive(0.05)
+        await resp.receive(timeout=0.05)
+
+    await resp.close()
+
+
+async def test_receive_timeout_deprecation(aiohttp_client) -> None:
+
+    async def handler(request):
+        ws = web.WebSocketResponse()
+        await ws.prepare(request)
+        await ws.receive()
+        await ws.close()
+        return ws
+
+    app = web.Application()
+    app.router.add_route("GET", "/", handler)
+
+    client = await aiohttp_client(app)
+    with pytest.warns(
+        DeprecationWarning,
+        match="float parameter 'receive_timeout' "
+        "is deprecated, please use parameter "
+        r"'timeout=ClientWSTimeout\(ws_receive=...\)'",
+    ):
+        resp = await client.ws_connect("/", receive_timeout=0.1)
+
+    with pytest.raises(asyncio.TimeoutError):
+        await resp.receive(timeout=0.05)
 
     await resp.close()
 
@@ -619,11 +755,16 @@ async def test_heartbeat_connection_closed(aiohttp_client: AiohttpClient) -> Non
     # since if we closed the connection normally, the client would
     # would cancel the heartbeat task and we wouldn't get a ping
     assert resp._conn is not None
-    with mock.patch.object(
-        resp._conn.transport, "write", side_effect=ClientConnectionResetError
-    ), mock.patch.object(resp._writer, "ping", wraps=resp._writer.ping) as ping:
+    with (
+        mock.patch.object(
+            resp._conn.transport, "write", side_effect=ClientConnectionResetError
+        ),
+        mock.patch.object(
+            resp._writer, "send_frame", wraps=resp._writer.send_frame
+        ) as send_frame,
+    ):
         await resp.receive()
-        ping_count = ping.call_count
+        ping_count = send_frame.call_args_list.count(mock.call(b"", WSMsgType.PING))
     # Connection should be closed roughly after 1.5x heartbeat.
     await asyncio.sleep(0.2)
     assert ping_count == 1
@@ -736,7 +877,10 @@ async def test_heartbeat_no_pong_concurrent_receive(
     async def handler(request):
         nonlocal ping_received
         ws = web.WebSocketResponse(autoping=False)
-        await ws.prepare(request)
+        with mock.patch(
+            "aiohttp.web_ws.WebSocketDataQueue", PatchableWebSocketDataQueue
+        ):
+            await ws.prepare(request)
         msg = await ws.receive()
         ping_received = msg.type is aiohttp.WSMsgType.PING
         ws._reader.feed_eof = lambda: None
@@ -745,21 +889,23 @@ async def test_heartbeat_no_pong_concurrent_receive(
     app = web.Application()
     app.router.add_route("GET", "/", handler)
 
-    client = await aiohttp_client(app)
-    resp = await client.ws_connect("/", heartbeat=0.1)
-    resp._reader.feed_eof = lambda: None
-
-    # Connection should be closed roughly after 1.5x heartbeat.
-    msg = await resp.receive(5.0)
-    assert ping_received
-    assert resp.close_code is WSCloseCode.ABNORMAL_CLOSURE
-    assert msg
-    assert msg.type is WSMsgType.ERROR
-    assert isinstance(msg.data, ServerTimeoutError)
+    with mock.patch("aiohttp.client.WebSocketDataQueue", PatchableWebSocketDataQueue):
+        client = await aiohttp_client(app)
+        resp = await client.ws_connect("/", heartbeat=0.1)
+    with mock.patch.object(
+        resp._reader, "feed_eof", autospec=True, spec_set=True, return_value=None
+    ):
+        # Connection should be closed roughly after 1.5x heartbeat.
+        msg = await resp.receive(5.0)
+        assert ping_received
+        assert resp.close_code is WSCloseCode.ABNORMAL_CLOSURE
+        assert msg.type is WSMsgType.ERROR
+        assert isinstance(msg.data, ServerTimeoutError)
+        assert str(msg.data) == "No PONG received after 0.05 seconds"
 
 
 async def test_close_websocket_while_ping_inflight(
-    aiohttp_client: AiohttpClient,
+    aiohttp_client: AiohttpClient, loop: asyncio.AbstractEventLoop
 ) -> None:
     """Test closing the websocket while a ping is in-flight."""
     ping_received = False
@@ -783,23 +929,27 @@ async def test_close_websocket_while_ping_inflight(
     await resp.send_bytes(b"ask")
 
     cancelled = False
-    ping_stated = False
+    ping_started = loop.create_future()
 
-    async def delayed_ping() -> None:
-        nonlocal cancelled, ping_stated
-        ping_stated = True
+    async def delayed_send_frame(
+        message: bytes, opcode: int, compress: Optional[int] = None
+    ) -> None:
+        assert opcode == WSMsgType.PING
+        nonlocal cancelled
+        ping_started.set_result(None)
         try:
             await asyncio.sleep(1)
         except asyncio.CancelledError:
             cancelled = True
             raise
 
-    with mock.patch.object(resp._writer, "ping", delayed_ping):
-        await asyncio.sleep(0.1)
+    with mock.patch.object(resp._writer, "send_frame", delayed_send_frame):
+        async with async_timeout.timeout(1):
+            await ping_started
 
     await resp.close()
     await asyncio.sleep(0)
-    assert ping_stated is True
+    assert ping_started.result() is None
     assert cancelled is True
 
 
@@ -1056,3 +1206,71 @@ async def test_ws_connect_with_wrong_ssl_type(aiohttp_client: AiohttpClient) ->
 
     with pytest.raises(TypeError, match="ssl should be SSLContext, .*"):
         await session.ws_connect("/", ssl=42)
+
+
+async def test_websocket_connection_not_closed_properly(
+    aiohttp_client: AiohttpClient,
+) -> None:
+    """Test that closing the connection via __del__ does not raise an exception."""
+
+    async def handler(request: web.Request) -> NoReturn:
+        ws = web.WebSocketResponse()
+        await ws.prepare(request)
+        await ws.close()
+        assert False
+
+    app = web.Application()
+    app.router.add_route("GET", "/", handler)
+    client = await aiohttp_client(app)
+    resp = await client.ws_connect("/")
+    assert resp._conn is not None
+    # Simulate the connection not being closed properly
+    # https://github.com/aio-libs/aiohttp/issues/9880
+    resp._conn.release()
+
+    # Clean up so the test does not leak
+    await resp.close()
+
+
+async def test_websocket_connection_cancellation(
+    aiohttp_client: AiohttpClient, loop: asyncio.AbstractEventLoop
+) -> None:
+    """Test canceling the WebSocket connection task does not raise an exception in __del__."""
+
+    async def handler(request: web.Request) -> NoReturn:
+        ws = web.WebSocketResponse()
+        await ws.prepare(request)
+        await ws.close()
+        assert False
+
+    app = web.Application()
+    app.router.add_route("GET", "/", handler)
+
+    sync_future: "asyncio.Future[List[aiohttp.ClientWebSocketResponse]]" = (
+        loop.create_future()
+    )
+    client = await aiohttp_client(app)
+
+    async def websocket_task() -> None:
+        resp = await client.ws_connect("/")
+        assert resp is not None  # ensure we hold a reference to the response
+        # The test harness will cleanup the unclosed websocket
+        # for us, so we need to copy the websockets to ensure
+        # we can control the cleanup
+        sync_future.set_result(client._websockets.copy())
+        client._websockets.clear()
+        await asyncio.sleep(0)
+
+    task = loop.create_task(websocket_task())
+    websockets = await sync_future
+    task.cancel()
+    with pytest.raises(asyncio.CancelledError):
+        await task
+
+    websocket = websockets.pop()
+    # Call the `__del__` methods manually since when it gets gc'd it not reproducible
+    del websocket._response
+
+    # Cleanup properly
+    websocket._response = mock.Mock()
+    await websocket.close()
