@@ -11,7 +11,7 @@ import pytest
 from yarl import URL
 
 import aiohttp
-from aiohttp.client_reqrep import ClientRequest, ClientResponse
+from aiohttp.client_reqrep import ClientRequest, ClientResponse, Fingerprint
 from aiohttp.connector import _SSL_CONTEXT_VERIFIED
 from aiohttp.helpers import TimerNoop
 from aiohttp.test_utils import make_mocked_coro
@@ -207,6 +207,7 @@ class TestProxy(unittest.TestCase):
         "aiohttp.connector.aiohappyeyeballs.start_connection",
         autospec=True,
         spec_set=True,
+        return_value=mock.create_autospec(socket.socket, spec_set=True, instance=True),
     )
     def test_proxy_connection_error(self, start_connection: Any) -> None:
         async def make_conn():
@@ -378,6 +379,115 @@ class TestProxy(unittest.TestCase):
         proxy_resp.close()
         self.loop.run_until_complete(req.close())
 
+    @mock.patch("aiohttp.connector.ClientRequest")
+    @mock.patch(
+        "aiohttp.connector.aiohappyeyeballs.start_connection",
+        autospec=True,
+        spec_set=True,
+    )
+    @pytest.mark.usefixtures("enable_cleanup_closed")
+    def test_https_connect_fingerprint_mismatch(
+        self, start_connection: mock.Mock, ClientRequestMock: mock.Mock
+    ) -> None:
+        async def make_conn() -> aiohttp.TCPConnector:
+            return aiohttp.TCPConnector(enable_cleanup_closed=cleanup)
+
+        for cleanup in (True, False):
+            with self.subTest(cleanup=cleanup):
+                proxy_req = ClientRequest(
+                    "GET", URL("http://proxy.example.com"), loop=self.loop
+                )
+                ClientRequestMock.return_value = proxy_req
+
+                class TransportMock(asyncio.Transport):
+                    def close(self) -> None:
+                        pass
+
+                proxy_resp = ClientResponse(
+                    "get",
+                    URL("http://proxy.example.com"),
+                    request_info=mock.Mock(),
+                    writer=mock.Mock(),
+                    continue100=None,
+                    timer=TimerNoop(),
+                    traces=[],
+                    loop=self.loop,
+                    session=mock.Mock(),
+                )
+                fingerprint_mock = mock.Mock(spec=Fingerprint, auto_spec=True)
+                fingerprint_mock.check.side_effect = aiohttp.ServerFingerprintMismatch(
+                    b"exp", b"got", "example.com", 8080
+                )
+                with (
+                    mock.patch.object(
+                        proxy_req,
+                        "send",
+                        autospec=True,
+                        spec_set=True,
+                        return_value=proxy_resp,
+                    ),
+                    mock.patch.object(
+                        proxy_resp,
+                        "start",
+                        autospec=True,
+                        spec_set=True,
+                        return_value=mock.Mock(status=200),
+                    ),
+                ):
+                    connector = self.loop.run_until_complete(make_conn())
+                    host = [
+                        {
+                            "hostname": "hostname",
+                            "host": "127.0.0.1",
+                            "port": 80,
+                            "family": socket.AF_INET,
+                            "proto": 0,
+                            "flags": 0,
+                        }
+                    ]
+                    with (
+                        mock.patch.object(
+                            connector,
+                            "_resolve_host",
+                            autospec=True,
+                            spec_set=True,
+                            return_value=host,
+                        ),
+                        mock.patch.object(
+                            connector,
+                            "_get_fingerprint",
+                            autospec=True,
+                            spec_set=True,
+                            return_value=fingerprint_mock,
+                        ),
+                        mock.patch.object(  # Called on connection to http://proxy.example.com
+                            self.loop,
+                            "create_connection",
+                            autospec=True,
+                            spec_set=True,
+                            return_value=(mock.Mock(), mock.Mock()),
+                        ),
+                        mock.patch.object(  # Called on connection to https://www.python.org
+                            self.loop,
+                            "start_tls",
+                            autospec=True,
+                            spec_set=True,
+                            return_value=TransportMock(),
+                        ),
+                    ):
+                        req = ClientRequest(
+                            "GET",
+                            URL("https://www.python.org"),
+                            proxy=URL("http://proxy.example.com"),
+                            loop=self.loop,
+                        )
+                        with self.assertRaises(aiohttp.ServerFingerprintMismatch):
+                            self.loop.run_until_complete(
+                                connector._create_connection(
+                                    req, [], aiohttp.ClientTimeout()
+                                )
+                            )
+
     @mock.patch("aiohttp.connector.ClientRequest")
     @mock.patch(
         "aiohttp.connector.aiohappyeyeballs.start_connection",
