@@ -1,5 +1,7 @@
 import datetime
+import logging
 import platform
+from typing import Any
 from unittest import mock
 
 import pytest
@@ -7,7 +9,7 @@ import pytest
 import aiohttp
 from aiohttp import web
 from aiohttp.abc import AbstractAccessLogger
-from aiohttp.helpers import PY_37
+from aiohttp.pytest_plugin import AiohttpClient, AiohttpServer
 from aiohttp.typedefs import Handler
 from aiohttp.web_log import AccessLogger
 
@@ -49,15 +51,48 @@ def test_access_logger_format() -> None:
     Ref: https://bitbucket.org/pypy/pypy/issues/1187/call-to-isinstance-in-__sub__-self-other
     Ref: https://github.com/celery/celery/issues/811
     Ref: https://stackoverflow.com/a/46102240/595220
-    """,  # noqa: E501
+    """,
 )
-def test_access_logger_atoms(mocker) -> None:
-    utcnow = datetime.datetime(1843, 1, 1, 0, 30)
-    mock_datetime = mocker.patch("datetime.datetime")
-    mock_getpid = mocker.patch("os.getpid")
-    mock_datetime.utcnow.return_value = utcnow
-    mock_getpid.return_value = 42
-    log_format = '%a %t %P %r %s %b %T %Tf %D "%{H1}i" "%{H2}i"'
+@pytest.mark.parametrize(
+    "log_format,expected,extra",
+    [
+        (
+            "%t",
+            "[01/Jan/1843:00:29:56 +0800]",
+            {"request_start_time": "[01/Jan/1843:00:29:56 +0800]"},
+        ),
+        (
+            '%a %t %P %r %s %b %T %Tf %D "%{H1}i" "%{H2}i"',
+            (
+                "127.0.0.2 [01/Jan/1843:00:29:56 +0800] <42> "
+                'GET /path HTTP/1.1 200 42 3 3.141593 3141593 "a" "b"'
+            ),
+            {
+                "first_request_line": "GET /path HTTP/1.1",
+                "process_id": "<42>",
+                "remote_address": "127.0.0.2",
+                "request_start_time": "[01/Jan/1843:00:29:56 +0800]",
+                "request_time": "3",
+                "request_time_frac": "3.141593",
+                "request_time_micro": "3141593",
+                "response_size": 42,
+                "response_status": 200,
+                "request_header": {"H1": "a", "H2": "b"},
+            },
+        ),
+    ],
+)
+def test_access_logger_atoms(
+    monkeypatch: Any, log_format: Any, expected: Any, extra: Any
+) -> None:
+    class PatchedDatetime(datetime.datetime):
+        @staticmethod
+        def now(tz):
+            return datetime.datetime(1843, 1, 1, 0, 30, tzinfo=tz)
+
+    monkeypatch.setattr("datetime.datetime", PatchedDatetime)
+    monkeypatch.setattr("time.timezone", -28800)
+    monkeypatch.setattr("os.getpid", lambda: 42)
     mock_logger = mock.Mock()
     access_logger = AccessLogger(mock_logger, log_format)
     request = mock.Mock(
@@ -69,23 +104,7 @@ def test_access_logger_atoms(mocker) -> None:
     )
     response = mock.Mock(headers={}, body_length=42, status=200)
     access_logger.log(request, response, 3.1415926)
-    assert not mock_logger.exception.called
-    expected = (
-        "127.0.0.2 [01/Jan/1843:00:29:56 +0000] <42> "
-        'GET /path HTTP/1.1 200 42 3 3.141593 3141593 "a" "b"'
-    )
-    extra = {
-        "first_request_line": "GET /path HTTP/1.1",
-        "process_id": "<42>",
-        "remote_address": "127.0.0.2",
-        "request_start_time": "[01/Jan/1843:00:29:56 +0000]",
-        "request_time": "3",
-        "request_time_frac": "3.141593",
-        "request_time_micro": "3141593",
-        "response_size": 42,
-        "response_status": 200,
-        "request_header": {"H1": "a", "H2": "b"},
-    }
+    assert not mock_logger.exception.called, mock_logger.exception.call_args
 
     mock_logger.info.assert_called_with(expected, extra=extra)
 
@@ -170,7 +189,6 @@ def test_logger_abc() -> None:
     mock_logger.info.assert_called_with("request response 1")
 
 
-@pytest.mark.skipif(not PY_37, reason="contextvars support is required")
 async def test_contextvars_logger(aiohttp_server, aiohttp_client):
     VAR = ContextVar("VAR")
 
@@ -196,3 +214,44 @@ async def test_contextvars_logger(aiohttp_server, aiohttp_client):
     resp = await client.get("/")
     assert 200 == resp.status
     assert msg == "contextvars: uuid"
+
+
+def test_access_logger_feeds_logger(caplog: pytest.LogCaptureFixture) -> None:
+    """Test that the logger still works."""
+    mock_logger = logging.getLogger("test.aiohttp.log")
+    mock_logger.setLevel(logging.INFO)
+    access_logger = AccessLogger(mock_logger, "%b")
+    access_logger.log(
+        mock.Mock(name="mock_request"), mock.Mock(name="mock_response"), 42
+    )
+    assert "mock_response" in caplog.text
+
+
+async def test_logger_does_not_log_when_not_enabled(
+    aiohttp_server: AiohttpServer,
+    aiohttp_client: AiohttpClient,
+    caplog: pytest.LogCaptureFixture,
+) -> None:
+    """Test logger does nothing when not enabled."""
+
+    async def handler(request: web.Request) -> web.Response:
+        return web.Response()
+
+    class Logger(AbstractAccessLogger):
+
+        def log(
+            self, request: web.BaseRequest, response: web.StreamResponse, time: float
+        ) -> None:
+            self.logger.critical("This should not be logged")  # pragma: no cover
+
+        @property
+        def enabled(self) -> bool:
+            return False
+
+    app = web.Application()
+    app.router.add_get("/", handler)
+    server = await aiohttp_server(app, access_log_class=Logger)
+    client = await aiohttp_client(server)
+    resp = await client.get("/")
+    assert 200 == resp.status
+    assert "This should not be logged" not in caplog.text
