@@ -4,7 +4,7 @@ from aiohttp import web
 from aiohttp.test_utils import make_mocked_coro
 
 
-async def serve(request):
+async def serve(request: web.BaseRequest) -> web.Response:
     return web.Response()
 
 
@@ -14,17 +14,18 @@ async def test_repr() -> None:
 
     assert "<RequestHandler disconnected>" == repr(handler)
 
-    handler.transport = object()
-    assert "<RequestHandler connected>" == repr(handler)
+    with mock.patch.object(handler, "transport", autospec=True):
+        assert "<RequestHandler connected>" == repr(handler)
 
 
 async def test_connections() -> None:
     manager = web.Server(serve)
     assert manager.connections == []
 
-    handler = object()
+    handler = mock.Mock(spec_set=web.RequestHandler)
+    handler._task_handler = None
     transport = object()
-    manager.connection_made(handler, transport)
+    manager.connection_made(handler, transport)  # type: ignore[arg-type]
     assert manager.connections == [handler]
 
     manager.connection_lost(handler, None)
@@ -34,7 +35,8 @@ async def test_connections() -> None:
 async def test_shutdown_no_timeout() -> None:
     manager = web.Server(serve)
 
-    handler = mock.Mock()
+    handler = mock.Mock(spec_set=web.RequestHandler)
+    handler._task_handler = None
     handler.shutdown = make_mocked_coro(mock.Mock())
     transport = mock.Mock()
     manager.connection_made(handler, transport)
